package com.persagy.netty.client;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.constant.CommonConst;
import com.persagy.entity.AlarmDefine;
import com.persagy.entity.NettyMessage;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.job.NettyMessageQueue;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.service.AlarmDefineService;
import com.persagy.utils.LockUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
public class CenterClientHandler extends SimpleChannelInboundHandler<NettyMessage<JSONObject>> {
    // Sleep 5 seconds before a reconnection attempt.
    static final int RECONNECT_DELAY = Integer.parseInt(System.getProperty("reconnectDelay", "5"));
    // Reconnect when the server sends nothing for 10 seconds.
    private static final int READ_TIMEOUT = Integer.parseInt(System.getProperty("readTimeout", "10"));

    private final AlarmDefineService alarmDefineService;
    private final NettyClient nettyClient;
    private final AlarmRecordRepository alarmRecordRepository;

    public CenterClientHandler(AlarmDefineService alarmDefineService, NettyClient nettyClient, AlarmRecordRepository alarmRecordRepository) {
        this.alarmDefineService = alarmDefineService;
        this.nettyClient = nettyClient;
        this.alarmRecordRepository = alarmRecordRepository;
    }

    /**
     * 在到服务器的连接已经建立之后将被调用
     * 当客户端主动链接服务端的链接后，这个通道就是活跃的了。也就是客户端与服务端建立了通信通道并且可以传输数据
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Connected to: {}", ctx.channel().remoteAddress());

        //建立连接
        nettyClient.sendMessage(new NettyMessage(200));

        //启动的时候发送消息,获取全部报警定义
        NettyMessage nettyMessage = new NettyMessage(4);
        JSONObject content = new JSONObject();
        content.put("groupCode", CommonConst.groupCode);
        content.put("projectId", CommonConst.projectId);
        nettyMessage.setContent(Arrays.asList(content));
        nettyClient.sendMessage(nettyMessage);

        //内存中缓冲的未发送成功的数据发送出去
        iniAlarmResult(ctx);
    }

    private void iniAlarmResult(ChannelHandlerContext ctx) {
        try {
            TimeInterval timer = DateUtil.timer();
            while (NettyMessageQueue.getNettyMessageQueue().size() > 0 && timer.interval() < 10000) {
                NettyMessage msg = NettyMessageQueue.getNettyMessageQueue().consume();
                log.info("剩余报警消息令总数:{}", NettyMessageQueue.getNettyMessageQueue().size());
                nettyClient.sendMessage(msg);
            }
        } catch (Exception e) {
            log.error("发送报警消息失败", e);
        }
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, NettyMessage msg) throws Exception {
        //接收msg消息{与上一章节相比，此处已经不需要自己进行解码}
        log.info("Client received: {}", msg);
        try {
            TimeInterval timer = DateUtil.timer();
            handlerMsg(ctx, msg);
            log.info("处理消息时间[{}]", timer.interval());
        } catch (Exception e) {
            log.error("channelRead", e);
        }
    }

    private void handlerMsg(ChannelHandlerContext channelHandlerContext, NettyMessage msg) throws Exception {
        if (msg.getOpCode() == 7) {
            log.info("--报警定义新增或更新--{}",msg);
            NettyMessage<AlarmDefine> AlarmDefineMessage = JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<AlarmDefine>>() {
            });
            List<AlarmDefine> definesList = AlarmDefineMessage.getContent();
            if (CollectionUtil.isNotEmpty(definesList)) {
                alarmDefineService.listSomeAlarmDefine(definesList);
            }
        } else if (msg.getOpCode() == 8) {
            log.info("-----报警记录id推送----[{}]", msg);
            List content = msg.getContent();
            if (CollectionUtil.isNotEmpty(content)) {
                JSONObject parseObject = JSONObject.parseObject(JSONObject.toJSONString(content.get(0)));
                String defineId = AlarmInfoCache.getAlarmDefineId(parseObject);
                ZktAlarmRecordDO zktAlarmRecordDO = alarmRecordRepository.findById(defineId).orElse(new ZktAlarmRecordDO());
                zktAlarmRecordDO.setDefinitionId(defineId);
                zktAlarmRecordDO.setObjId(parseObject.getString("objId"));
                zktAlarmRecordDO.setItemId(parseObject.getString("itemId"));
                zktAlarmRecordDO.setAlarmId(parseObject.getString("id"));
                alarmRecordRepository.save(zktAlarmRecordDO);
            }
        } else if (msg.getOpCode() == 9) {
            NettyMessage<AlarmDefine> AlarmDefineMessage = JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<AlarmDefine>>() {
            });
            List<AlarmDefine> definesList = AlarmDefineMessage.getContent();
            if (CollectionUtil.isNotEmpty(definesList)) {
                try {
                    LockUtil.getInstance().lock.lock();
                    LockUtil.getInstance().setExecute(false);
                    //加个等待，保证正在执行的逻辑执行成功
                    Thread.sleep(4000);
                    alarmDefineService.listAllAlarmDefine(definesList);
                    LockUtil.getInstance().setExecute(true);
                    LockUtil.getInstance().condition.signalAll();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                } finally {
                    LockUtil.getInstance().lock.unlock();
                }
            }
        } else if (msg.getOpCode() == 10) {
            NettyMessage<AlarmDefine> AlarmDefineMessage = JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<AlarmDefine>>() {
            });
            List<AlarmDefine> definesList = AlarmDefineMessage.getContent();
            if (CollectionUtil.isNotEmpty(definesList)) {
                alarmDefineService.deleteAlarmDefine(definesList);
            }
        } else if (msg.getOpCode() == 11) {
            //更新隔离的系统对象
            NettyMessage<String> AlarmDefineMessage = JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<String>>() {
            });
            List<String> isolationSystemList = AlarmDefineMessage.getContent();
            if (CollectionUtil.isNotEmpty(isolationSystemList)) {
                AlarmInfoCache.isolationSystemList = isolationSystemList;
            }
        } else if (msg.getOpCode() == 12) {
            //云端更新报警记录状态
            NettyMessage<JSONObject> AlarmStateMessage = JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<JSONObject>>() {});
            List<JSONObject> stateList = AlarmStateMessage.getContent();
            alarmDefineService.updateAlarmDefine(stateList);
        }
        NettyMessage response = new NettyMessage(3);
        response.setRemark("已经收到消息");
        nettyClient.sendMessage(response);
    }

    /**
     * 在处理过程中引发异常时被调用
     * 抓住异常，当发生异常的时候，可以做一些相应的处理，比如打印日志、关闭链接
     *
     * @param ctx
     * @param cause
     * @throws Exception
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.info("exceptionCaught", cause);
        ctx.close();
        throw new Exception(cause);
    }

    /**
     * 通道处于非活跃状态动作，该方法只会在失效时调用一次
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        //客户端自己不正常情况下自己在重连一次
        log.info("Disconnected from: " + ctx.channel().remoteAddress());

    }


    @Override
    public void channelUnregistered(final ChannelHandlerContext ctx) throws Exception {
        log.info("channelUnregistered and reconnecting to: {}:{} ", RECONNECT_DELAY, CommonConst.inetHost, CommonConst.inetPort);
        ctx.channel().eventLoop().schedule(new Runnable() {
            @Override
            public void run() {
                nettyClient.reConnect();
            }
        }, RECONNECT_DELAY, TimeUnit.SECONDS);
    }

}
