package com.persagy.service.impl;

import cn.hutool.core.date.DateUtil;
import com.persagy.job.AlarmExpireJob;
import com.persagy.service.AlarmQuartzService;
import com.persagy.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashSet;

import static org.quartz.SimpleScheduleBuilder.simpleSchedule;

/**
 * @description:报警定时任务服务
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/9/18 001820:11
 * @version:V1.0
 **/
@Slf4j
@Service
public class AlarmQuartzServiceImpl implements AlarmQuartzService {

    @Autowired
    @Qualifier("quartzScheduler")
    Scheduler quartzScheduler;


    /**
     * 添加定时器
     *
     * @param startTime    添加报警恢复定时任务
     * @param jobName      job名称
     * @param jobGroupName job分组
     * @param jobDataMap   触发器的数据（任务相关的数据都放在这里）
     * @return
     * @throws SchedulerException
     */
    @Override
    public String addExpireJob(Date startTime, String jobName, String jobGroupName, JobDataMap jobDataMap) throws SchedulerException {
        if (startTime.before(new Date())) {
            log.error("执行时间【{}】为历史时间，继续添加", DateUtils.formatDate(startTime));
        }
        JobKey jobKey = new JobKey(jobName, jobGroupName);
        JobDetail jobDetail = JobBuilder.newJob(AlarmExpireJob.class).withIdentity(jobKey).requestRecovery().build();
        Trigger trigger = TriggerBuilder.newTrigger().startAt(startTime)
                .withIdentity("trigger_" + jobName, jobGroupName)
                .usingJobData(jobDataMap)
                .withSchedule(simpleSchedule().withMisfireHandlingInstructionIgnoreMisfires()).build();
        //已经存在的job实例和触发器自动覆盖 job实 例 唯 一标识：
        HashSet<Trigger> triggerSet = new HashSet<Trigger>();
        Date fireTime = trigger.getFireTimeAfter(DateUtil.offsetMinute(startTime, -1).toJdkDate());
        if (fireTime != null) {
            log.info("执行时间为:【{}】，设置时间为【{}】", DateUtils.formatDate(fireTime), DateUtils.formatDate(startTime));
        } else {
            log.info("执行时间为 空!");
        }
        triggerSet.add(trigger);
        quartzScheduler.scheduleJob(jobDetail, triggerSet, true);
        return "success";
    }


    /**
     * @param jobName:job名称
     * @param jobGroupName:job分组名称
     * @description:删除定时任务
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @return: java.lang.String
     * @since: 2020/09/29 9:23
     * @version: V1.0
     */
    @Override
    public synchronized String deleteExpireJob(String jobName, String jobGroupName) throws SchedulerException {
        JobKey jobKey = new JobKey(jobName, jobGroupName);
        boolean checkExists = quartzScheduler.checkExists(jobKey);
        if (checkExists) {
            quartzScheduler.deleteJob(jobKey);
        }
        return "success!";
    }


}
